# -*- coding: utf-8 -*-
require 'stringio'

class Card < ActiveRecord::Base
  belongs_to :language

  validates_presence_of :language, :message => N_("%{fn} must be selected.")
  validates_presence_of :given_name, :message => N_("%{fn} cannot be blank.")

  file_column :photo, :magick => {
    :geometry => '480x480>',
    :versions => {
      'icon' => '20x20>',
      'small' => '50x50>',
    },
  }
  validates_filesize_of :photo, :in => 0..3.megabyte, :message => N_("Size of %{fn} is too huge.")

  def self.all_sorted(*args)
    with_scope(:find => { :order => 'sort_name, family_name, given_name, additional_name' }) {
      all(*args)
    }
  end

  def full_name
    case language.name
    when 'ja'
      [
        family_name, additional_name, given_name
      ].reject { |x| x.blank? }.join(' ')
    else
      [
        given_name, additional_name, family_name
      ].reject { |x| x.blank? }.join(' ')
    end
  end

  def delete_photo
    if @delete_photo.nil?
      false
    else
      @delete_photo
    end
  end

  def delete_photo=(value)
    if value == '1'
      self.photo = nil
      save!
      @delete_photo = true
    else
      @delete_photo = false
    end
  end

  def to_vcard
    return Vpim::Vcard::Maker.make2 { |maker|
      maker.add_name { |name|
        name.given = given_name
        name.family = family_name
        name.additional = additional_name
        name.fullname = full_name
      }
      maker.nickname = nickname unless nickname.blank?
      if !sort_name.blank?
        maker.add_field(Vpim::DirectoryInfo::Field.create('SORT-STRING', sort_name))
        family, given = sort_name.split(' ', 2)
        maker.add_field(Vpim::DirectoryInfo::Field.create('X-PHONETIC-LAST-NAME', family)) if family
        maker.add_field(Vpim::DirectoryInfo::Field.create('X-PHONETIC-FIRST-NAME', given)) if given
      end
      maker.add_tel(telephone) { |x|
        x.location = 'home'
        x.preferred = true
      } unless telephone.blank?
      maker.add_email(email) { |x|
        x.location = 'home'
        x.preferred = true
      } unless email.blank?
      maker.add_photo { |x|
        image = Magick::Image.read(photo.to_s).first
        image.format = 'jpeg'
        x.type = 'jpeg'
        x.image = image.to_blob
      } unless photo.blank?
    }
  end

  def self.new_from_vcard(vcard)
    returning(Card.new) { |card|
      returning(vcard.name) { |names|
        card.language_id = Language.guess_from_string(names.given).id
        card.given_name = names.given
        card.family_name = names.family
        card.additional_name = names.additional
        card.nickname = vcard.nickname
        if !(sort_name = vcard['SORT-STRING']).blank? ||
            !(sort_name = [
              vcard['X-PHONETIC-LAST-NAME'],
              vcard['X-PHONETIC-FIRST-NAME'],
            ].compact.join(' ')).blank?
          card.sort_name = sort_name
        end
      }
      telephone = vcard.telephones.find { |telephone|
        telephone.preferred
      } || vcard.telephone
      card.telephone = telephone.to_s unless telephone.blank?
      email = vcard.emails.find { |email|
        email.preferred
      } || vcard.email
      card.email = email.to_s unless email.blank?
      vcard.photos.each { |photo|
        mime_type = format_to_mime_type(photo.format)
        file = StringIO.new(photo.to_s)
        class << file
          attr_reader :original_filename
          attr_reader :content_type
        end
        file.instance_eval {
          @original_filename = 'photo.' + mime_type.extensions.first
          @content_type = mime_type.content_type
        }
        card.photo = file
        break
      }
    }
  end

  def self.format_to_mime_type(format)
    m = format.match(%r{(.*/.*)|(.*)})
    type_name = m[1] || 'image/' + m[2]
    MIME::Types[type_name].first || MIME::Types['image/jpeg'].first
  end
end
